%Power ratings used for calculating the mass and cost of the vehicle
P_EM_rat        = P_EM_max;                                 %[W] Electric machine peak power
P_Inv_rat       = V_Bat_max_cell*N_Bat_s*max(abs(i_Inv));   %[VA] Peak inverter VA rating
p_BR            = v_Bat.*i_BR;                              %[W] Power of braking resistor
P_BR_rat        = max(p_BR);                                %[W] Rated braking resistor
E_BR            = trapz(t_sim,p_BR)/3600;                   %[Wh] Energy dissipated in the braking resistor

p_Charging      = i_BC.*v_Bat;                              %[W] Charging power to the battery
E_Charging      = trapz(t_sim,p_Charging)/3600;             %[Wh] Charging energy to the battery
p_t             = v_vehicle.*f_t;                           %[W] Traction power
E_t             = trapz(t_sim,p_t)/3600;                    %[Wh] Traction energy

p_Inv           = i_Inv.*v_Bat;                             %[W] Inverter power
E_Inv           = trapz(t_sim,p_Inv)/3600;                  %[Wh] Inverter energy
p_Aux           = v_Bat.*i_Aux;                             %[W] Auxilliary power
E_Aux           = trapz(t_sim,p_Aux)/3600;                  %[Wh] Energy of auxiliary devices

p_MB            = tau_MB.*w_s;                              %[W] Mechanics braking power
E_MB            = trapz(t_sim,p_MB)/3600;                   %[Wh] Energy dissipated in the mechanic brakes
p_s             = tau_s.*w_s;                               %[W] Shaft power
E_Loss_TS       = trapz(t_sim,abs(p_s-p_t))/3600;           %[Wh] Loss in the transmission system
E_Loss_EM       = trapz(t_sim,abs(p_EM-p_s))/3600;          %[Wh] Loss in the electric machine
E_Loss_Inv      = trapz(t_sim,abs(p_Inv-p_EM))/3600;        %[Wh] Loss in the inverter
p_Bat           = v_Bat.*i_Bat;                             %[W] Battery power
E_Loss_Bat      = abs(trapz(t_sim,p_Bat))/3600;             %[Wh] Loss in the battery

[N,~]           = size(v_Bat);                              %[-] Entries of v_Bat
V_Bat_ini_cell  = v_Bat(N)/N_Bat_s; clear N;                %[V] Initial battery cell voltage of the next simulation

switch vehicle
    case 'BEV'
        E_Grid          = trapz(t_sim,p_Grid)/3600;                 %[Wh] Grid energy
        E_Loss_BC       = trapz(t_sim,abs(p_RF-p_BC))/3600;         %[Wh] Loss in the boost converter
        E_Loss_RF       = trapz(t_sim,abs(p_Grid-p_RF))/3600;       %[Wh] Loss in the rectifier
    case 'FCHEV'
        E_H2            = trapz(t_sim,p_H2)/3600;                   %[Wh] Hydrogen energy
        E_Loss_BC       = trapz(t_sim,p_FC-p_BC)/3600;              %[Wh] Loss in the boost converter
        E_Loss_FC       = trapz(t_sim,p_H2-p_FC)/3600;              %[Wh] Loss in the fuel cell stack
        N_start         = find(t_sim >= T_Cycle_start,1,'first');   %[-] Start entry of drive cycle
        N_stop          = find(t_sim <= T_Cycle_stop,1,'last');     %[-] Stop entry of drive cycle
        P_FC_av         = trapz(t_sim(N_start:N_stop),p_Inv(N_start:N_stop)+p_Aux(N_start:N_stop))/(T_Cycle_stop-T_Cycle_start);  %[W] Average traction power
end