M_EM                = P_EM_rat/(1000*SP_EM);                                                            %[kg] Mass of electric machine
M_Inv               = P_Inv_rat/(1000*SP_Inv);                                                          %[kg] Mass of inverter
M_Bat               = M_Bat_cell*N_Bat_s*N_Bat_p;                                                       %[kg] Mass of battery package
M_BR                = P_BR_rat/(1000*SP_Inv);                                                           %[kg] Mass of braking resistor

Vol_EM              = P_EM_rat/(1000*PD_EM);                                                            %[L] Volume of electric machine
Vol_Inv             = P_Inv_rat/(1000*PD_Inv);                                                          %[L] Volume of inverter
Vol_Bat             = Vol_Bat_cell*N_Bat_s*N_Bat_p;                                                     %[L] Volume of battery
Vol_BR              = P_BR_rat/(1000*PD_Inv);                                                           %[L] Volume of braking resistor

Cost_EM             = 16*P_EM_rat/1000 + 385;                                                           %[$] Electric machine cost
Cost_Inv            = CP_Inv*P_Inv_rat/1000;                                                            %[$] Inverter cost
% PE_Bat              = I_Bat_max_cell/Q_Bat_1_cell;                                                    %[1/h] Peak power energy ratio of the battery
% CE_Bat              = 11.1*PE_Bat + 211.1;                                                            %[$] Battery cost per kWh
E_Bat               = N_Bat_s*V_Bat_nom_cell*N_Bat_p*Q_Bat_1_cell;                                      %[Wh] Total capacity of the battery
Cost_Bat            = CE_Bat*E_Bat/1000;                                                                %[$] Battery cost
Cost_BR             = CP_Inv*P_BR_rat/1000;                                                             %[$] Braking resistor cost

switch vehicle
    case 'BEV'
        M_BC        = P_BC_rat/(1000*SP_Inv);                                                           %[kg] Mass of boost converter
        M_RF        = P_RF_rat/(1000*SP_Inv);                                                           %[kg] Mass of active rectifier
        M_car       = M_glider + M_person + M_EM + M_Inv + M_Bat + M_BC + M_BR + M_RF;                  %[kg] Total mass of car
        
        Vol_BC      = P_BC_rat/(1000*PD_Inv);                                                           %[L] Volume of boost converter
        Vol_RF      = P_RF_rat/(1000*PD_Inv);                                                           %[L] Volume of active rectifier
        Vol_PS      = Vol_EM + Vol_Inv + Vol_Bat + Vol_BC + Vol_BR + Vol_RF;                            %[L] Volume of power system

        Cost_BC     = CP_Con*P_BC_rat/1000;                                                             %[$] Boost converter cost
        Cost_RF     = CP_Inv*P_RF_rat/1000;                                                             %[$] Rectifier cost
        Cost_car    = Cost_glider + Cost_EM + Cost_Inv + Cost_Bat + Cost_BC + Cost_BR + Cost_RF;        %[$] Total cost of car
    case 'FCHEV'
        slope_FC_max = slope_FC_max_cell*N_FC_p*N_FC_s;                                                 %[W/s] Maximum power slope
        P_FC_rat    = N_FC_s*N_FC_p*V_FC_nom_cell*I_FC_nom_cell;                                        %[W] Rated fuel cell power
        P_FC_max    = P_FC_rat;
        P_BC_rat    = P_FC_rat;                                                                         %[W] Rated boost converter power
        Cost_BC     = CP_Con*P_BC_rat/1000;                                                             %[$] Boost converter cost
        
        M_BC        = P_BC_rat/(1000*SP_Inv);                                                           %[kg] Mass of boost converter
        M_FC        = Mass_FC_cell*N_FC_p*N_FC_s;                                                       %[kg] Mass of fuel cell
        M_H2        = E_H2*3600/LHV_H2;                                                                 %[kg] Mass of hydrogen
        M_sto       = E_H2/(1000*SE_sto);                                                               %[kg] Mass of storage
        M_car       = M_glider + M_person + M_EM + M_Inv + M_Bat + M_BC + M_BR + M_FC + M_H2 + M_sto;   %[kg] Total mass of car
        
        Vol_BC      = P_BC_rat/(1000*PD_Inv);                                                           %[L] Volume of boost converter
        Vol_FC      = Vol_FC_cell*N_FC_p*N_FC_s;                                                        %[L] Volume of fuel cell
        Vol_sto     = E_H2/(1000*ED_sto);                                                               %[L] Volume of storage
        Vol_PS      = Vol_EM + Vol_Inv + Vol_Bat + Vol_BR + Vol_BC + Vol_FC + Vol_sto;                  %[L] Volume of power system

        Cost_FC     = CP_FC*P_FC_max/1000;                                                              %[$] Fuel cell cost
        Cost_sto    = CM_sto*M_H2;                                                                      %[$] Cost of hydrogen storage
        Cost_car    = Cost_glider + Cost_EM + Cost_Inv + Cost_Bat + Cost_BC + Cost_BR + Cost_FC + Cost_sto; %[$] Total cost of car
end

c_rr                = 0.01*(1 + v_car_dc/100);                              %[-] Coefficienct of roling friction
f_t_temp            = M_car*d_v_car_dc_dt + M_car*g*sin(alpha) + sign(v_car_dc/3.6)*M_car*g*cos(alpha).*c_rr...
    + sign(v_car_dc/3.6+V_wind)*0.5*rho_air*C_drag*A_front.*(v_car_dc/3.6 + V_wind).^2;                 %[N] Nominal traction force
f_t_req_max         = 1.05*max(f_t_temp); clear f_t_temp;

c_rr_max            = 0.01*(1 + 3.6*V_car_max/100);                         %[-] Coefficienct of roling friction
f_t_temp            = M_car*g*sin(alpha) + M_car*g*cos(alpha).*c_rr_max...
    + 0.5*rho_air*C_drag*A_front.*(V_car_max + V_wind).^2;                  %[N] Nominal traction force
f_t_req_nom         = 1.05*f_t_temp; clear f_t_temp;

f_t_max             = max([f_t_req_max f_t_req_nom*a_EM_2]);                %[N] Maximum traction force
tau_t_max           = f_t_max*r_w;                                          %[Nm] Maximu traction torque
tau_s_max           = tau_t_max/(G*eta_TS);                                 %[Nm] Nominal shaft torque
tau_s_cont          = tau_s_max/a_EM_1;                                     %[Nm] Nominal shaft torque

w_s_max_tau_s_cont  = w_s_max/a_EM_3;                                       %[rad/s] Nominal shaft speed
P_s_max             = w_s_max_tau_s_cont*tau_s_cont;                        %[W] Maximum shaft power
tau_c               = (0.02/w_s_max)*(P_s_max/0.9);
B_v                 = (0.06/w_s_max^2)*(P_s_max/0.9);

w_e_max_tau_s_cont  = w_s_max_tau_s_cont*P/2;
tau_e_cont          = tau_c + B_v*w_s_max_tau_s_cont + tau_s_cont;          %[Nm] Cont. electro mechanical torque
m_i_nom             = 1/a_EM_3;                                             %[-] Maximum modulation index
V_p_peak_nom        = m_i_nom*V_Bus_min/2;                                  %[V] Nominal peak phase voltage
V_d_nom             = -V_p_peak_nom*sin(phi_EM_nom);                        %[V] Nominal d-axis voltage
V_q_nom             = V_p_peak_nom*cos(phi_EM_nom);                         %[V] Nominal q-axis voltage
P_EM_max            = P_s_max/eta_EM_nom;                                   %[W] Maximum electric input power
I_q_cont            = (2/3)*P_EM_max/V_q_nom;                               %[A] Nominal q-axis current
lambda_pm           = (2/3)*(2/P)*tau_e_cont/I_q_cont;                      %[Wb] Permanent magnet flux linkage
L_q                 = -V_d_nom/(w_e_max_tau_s_cont*I_q_cont);               %[H] D-axis inductance
R_s                 = (V_q_nom - w_e_max_tau_s_cont*lambda_pm)/I_q_cont;    %[Ohm] Stator resistance
if (lambda_pm <=0 || L_q <= 0 || R_s <= 0)
    disp('Warning: Negative electric machine parameters!')
end

P_Inv_loss_rat      = ((1-eta_Inv_nom)/eta_Inv_nom)*P_EM_max;                           %[W] Rated inverter loss
tau_e_max           = tau_c + B_v*(P_s_max/tau_s_max) + tau_s_max;                      %[Nm] Maximum electro mechanical torque
I_q_max             = (2/3)*(2/P)*tau_e_max/lambda_pm;                                  %[A] Maximum Q-axis current

R_Inv               = (P_Inv_loss_rat - (6/pi)*V_th_Inv*I_q_max)/((3/2)*I_q_max^2);     %[Ohm] Inverter switch resistance
if R_Inv <= 0
    disp('Warning: Negative inverter parameter!')
end

switch vehicle
    case 'BEV'
        R_BC                = (P_RF_max*(1 - eta_BC_nom) - V_th_BC*I_RF_max)/(I_RF_max^2);      %[Ohm] Swith resistance of boost converter
        if R_BC <= 0
            disp('Warning: Negative boost converter parameter!')
        end

        R_RF              = (P_Grid_max*(1 - eta_RF_nom) - 2*V_th_RF*I_RF_max)/(2*I_RF_max^2);  %[Ohm] Resistance of rectifier
        if R_RF <= 0
            disp('Warning: Negative rectifier parameter!')
        end
    case 'FCHEV'
        I_FC_max = N_FC_p*I_FC_nom_cell;                                                        %[A] Maximum fuel cell current
        R_BC                = (P_FC_max*(1 - eta_BC_nom) - V_th_BC*I_FC_max)/(I_FC_max^2);      %[Ohm] Swith resistance of boost converter
        if R_BC <= 0
            disp('Warning: Negative boost converter parameter!')
        end
end