%Vehicle parameters
M_glider    = 905;                                                          %[kg] Glider mass (without power train)
M_person    = 100;                                                          %[kg] Mass of one person
%r_w         = 0.2794;
%%[m] Wheel radius
r_w         = (16*0.0254+2*205*0.6*1e-3)/2;                                 %[m] Wheel radius of Toyota Avensis, 205/60R16
eta_TS      = 0.95;                                                         %[-] Transmission system efficiency
P_Aux       = 957;                                                          %[W] Electric auxiliary loads
V_car_max   = 200/3.6;                                                      %[m/s] Maximum vehicle speed

%Mass and volume ratings
SP_EM       = 1.83;                                                         %[kW/kg] Electric machine peak specific power of UQM system
PD_EM       = 4.83;                                                         %[kW/L] Electric machine peak power density of UQM system
SP_Inv      = 9.3;                                                          %[kW/kg] Inverter peak specific power of Toyota Camry
PD_Inv      = 11.7;                                                         %[kW/L] Inverter peak power density of Toyota Camry

%Cost ratings
Cost_glider = 12842;                                                        %[$] Glider cost (without power train)
CP_Inv      = 5;                                                            %[$/kW] Inverter cost per peak power
CP_Con      = 75;                                                           %[$/kW] DC/DC converter cost per peak power
CE_Bat      = 250;                                                          %[$/kWh] Battey cost

rho_air     = 1.2041;                                                       %[kg/m^3] Air density of dry air at 20 degree C
g           = 9.81;                                                         %[m/s^2] Free fall acceleration
C_drag      = 0.3;                                                          %[-] Aerodynamic drag coefficient
A_front     = 2.27;                                                         %[m^2] Front area
V_car       = 50/3.6;                                                       %[m/s] 50 km/h speed
V_wind      = 0;                                                            %[m/s] Wind speed
alpha       = 0;                                                            %[rad] Slope of road
%T_force_to_speed = tf([1], [1 (rho_air*C_drag*A_front*(V_car + V_wind)/M_car + 0.00036*g*cos(alpha))]); % Force-to-speed transfer function
%T_spee_con = 2*76.787*tf([8.2 1], [1 0]);   	% Speed controller

% figure(2)
% bode(T_force_to_speed,'r', T_spee_con*T_force_to_speed,'g')
% xlim([1e-4 1e4])
% legend('Force-to-speed', 'Error-to-speed')
% grid
% 
% print(2, '-depsc2', '-r50', '../../../CEESA/vehicle_energy_consumption/report/gfx/bode_plot.eps')

load('Saft_VL37570_LiIon.mat')
switch vehicle
    case 'FCHEV'
        load('Ballard_FCgen_1020ACS.mat')
end

%Battery design parameters
SoC_Bat_min = 0.2;                                                  %[-] Minimum state-of-charge
SoC_Bat_Ini = 0.9;                                                  %[-] Initial state-of-charge
DoD_Bat_Ini = 1 - SoC_Bat_Ini;                                      %[-] Initial depth-of-discharge

%Motor design parameters
P                   = 18;                                           %[-] Number of poles
V_Bus_nom           = 600;                                          %[V] Nominal bus voltage
eta_EM_nom          = 0.94;                                         %[-] Efficiency at nominal load and speed
phi_EM_nom          = 0.55;                                         %[rad] phase angel at nominal torque and speed
n_s_max             = 8000;                                         %[rpm] Maximum speed
w_s_max             = 2*pi*n_s_max/60;                              %[rad/s] Maximum angular shaft velocity
G                   = round(r_w*w_s_max/V_car_max);                 %[-] Gear ratio of differential
a_EM_1              = 240/150;                                      %[-] Relation ship between nominal and maximum shaft torque
a_EM_2              = 240/85;                                       %[-] Relation ship between maximum torque and maximum torqe at maximum speed
a_EM_3              = 8000/4775;                                    %[-] Relationship between maximum speed at continuous torque and maximum shaft speed

%Inverter design parameters
eta_Inv_nom         = 0.95;                                         %[-] Nominal inverter efficiency
V_th_Inv            = 1;                                            %[V] Threshold voltage of the switch and diode in an IGBT

%Boost converter design parameters
eta_BC_nom = 0.95;                                                  %[-] Nominal boost converter efficiency
V_th_BC    = 1;                                                     %[V] Threshold voltage

N_Bat_s             = round(V_Bus_nom/V_Bat_nom_cell);              %[-] Number of series cells
V_Bus_min           = N_Bat_s*V_Bat_min_cell;                       %[V] Minimum bus voltage

switch vehicle
    case 'BEV'
        %Rectifier design parameters
        V_LL                = 400;                                          %[V] Line-line rms voltage
        I_Grid_max          = 16;                                           %[A] Maximum rms line current
        P_Grid_max          = (3*sqrt(3)/pi)*V_LL*I_Grid_max;               %[W] Maximum grid power
        I_RF_max            = sqrt(3/2)*I_Grid_max;                         %[A] Maximum current of the rectifier
        V_RF                = 3*sqrt(2)*V_LL/pi;                            %[V] Rectified average voltage
        V_th_RF             = 1;                                            %[V] Threshold voltage
        eta_RF_nom          = 0.95;                                         %[-] Nominal efficiency of rectifier
        
        %Initial values used for the design procedure
        P_RF_max            = P_Grid_max;                                   %[VA] Rated rectifier VA-rating
        P_RF_rat            = P_RF_max;                                     %[W] Rated rectifier VA
        P_BC_rat            = P_RF_rat;                                     %[W] Rated boost-converter power
    case 'FCHEV'
        %Fuel cell design parameters
        F           = 96485;                                                %[C/mol] Faraday's constant
        M_H2_mol    = 0.00216;                                              %[kg/mol] Hydrogen molar mass
        LHV_H2      = 120.1e6;                                              %[J/kg] Lover heating value of hydrogen
        CP_FC       = 500;                                                  %[$/kW] Fuel cell cost
        CM_sto      = 700;                                                  %[$/kg] Cost of hydrogen storage per kg H2
        ED_sto      = 1.5;                                                  %[kWh/L] Energy density of hydroggen storage
        SE_sto      = 2;                                                    %[kWh/kg] Specific energy og hydrogen storage
        
        N_FC_s      = round(V_Bus_min/V_FC_max_cell);                       %[-] Number of series connected fuel cells
        
        %Initial values used for the design procedure
        E_H2  = 20000;                                                      %[Wh] Energy of used hydrogen
end

%Initial values used for the design procedure
N_Bat_p             = 10;                                           %[-] Number of parallel strings
P_EM_rat            = 100e3;                                        %[W] Electric machine peak power
P_Inv_rat           = 1.5*P_EM_rat;                                 %[W] Peak inverter power
P_BR_rat            = 1000;                                         %[W] Rated braking resistor
V_Bat_ini_cell      = V_Bat_nom_cell;                               %[V] Initial battery cell voltage