clear all
close all
%clc

disp('Options:')
disp('1: Battery Electric Vehicle (BEV)')
disp('2: Fuel Cell Hybrid Electric Vehicle (FCHEV)')
disp('0: Exit')
VN = 4;
while (VN~=0) && (VN~=1) && (VN~=2)
    VN = input('Press 0, 1, or 2 followed by Enter: ');
    switch VN
        case 1
            vehicle = 'BEV';
            disp('You have chosen the BEV')
        case 2
            vehicle = 'FCHEV';
            disp('You have chosen the FCHEV')
    end
end
clear VN;

%Load drive cycle
load('drive_cycle.mat')

figure(1)
plot(t,v_car,'.-')
title('Driving cycle')
xlabel('Time [s]')
ylabel('Vehicle speed [km/h]')
xlim([min(t) max(t)])
grid

while exist('N_dc','var')~=1
    N_dc = input('Press a real number higher than 0 followed by Enter in order to determine the number the driving cycle should be repeated: ');
end
if N_dc == 1
    disp(['You have chosen ', num2str(N_dc), ' repetition which means that the total driving distance is ', num2str(round(N_dc*D_max(4))), ' km'])
else
    disp(['You have chosen ', num2str(N_dc), ' repetitions which means that the total driving distance is ', num2str(round(N_dc*D_max(4))), ' km'])
end
disp('The vehicle simulation program is now sizing the main components of the vehicle and calculating the key figures of it. Please have patience...')

tic

%Initializtion routine
initialization

E_all        = zeros(1,10);
eta_all      = zeros(1,10);
FC_car       = zeros(1,10);
Mass_all     = zeros(1,10);
Vol_PS_all   = zeros(1,10);
Cost_all     = zeros(1,10);

for N_dc_design = N_dc:N_dc
    [T_s_sim, T_Cycle_start, T_Cycle_stop, T_start, T_stop, t_dc, v_car_dc, d_v_car_dc_dt] = number_of_dc(t, v_car, N_dc_design);
    
    %Initialization
    FC_ok       = 0;
    N_FC_p      = 1;
    i_ite_FC    = 1;
    M_eval      = zeros(1,4);
    
    while FC_ok ~= 1
        
        %Initialization
        Bat_ok = 0;                                                                            %[-] 1: Bat have the right size, 0: not
        
        %Algorithmen finds the minimum number of parallel strings N_Bat that makes
        %sure that both the energy requirement (min(SoC_Bat) >= 20%) and power
        %requirement (max(i_Bat) <= I_Bat_peak) are full filled
        i_ite = 1;
        M_E_P = zeros(1,5);
        N_Bat_p_under = 0;
        while Bat_ok ~= 1
            for i=1:3
                design
                switch vehicle
                    case 'BEV'
                        sim('BEV', [T_Cycle_start (T_Cycle_stop+100)], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
                    case 'FCHEV'
                        sim('FCHEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
                end
                PowerEnergyCalculation
                M_E_P(i_ite,1:3) = [N_Bat_p min(SoC_Bat) max(i_Bat/N_Bat_p)];
            end
            
            N_Bat_p_pre = N_Bat_p;
            if ((N_Bat_p*I_Bat_max_cell >= max(i_Bat)) && (SoC_Bat_min <= min(SoC_Bat)))            %Find smallest number N_Bat_p_over that insures that the energy and power requirements are fulfilled
                if (exist('N_Bat_p_over', 'var') == 1)
                    if (N_Bat_p < N_Bat_p_over)
                        N_Bat_p_over = N_Bat_p;
                    end
                else
                    N_Bat_p_over = N_Bat_p;
                end
            else                                                                                    %Find the highest number N_Bat_p_under, where either the energy or power requirement not are fulfilled
                if (N_Bat_p > N_Bat_p_under)
                    N_Bat_p_under = N_Bat_p;
                end
            end
            if (exist('N_Bat_p_over', 'var') == 1)        %Find the number N_Bat_p for the next iteration
                if ((N_Bat_p*I_Bat_max_cell >= max(i_Bat)) && (SoC_Bat_min <= min(SoC_Bat)))
                    if N_Bat_p_over - N_Bat_p_under ~= 1
                        N_Bat_p = round(N_Bat_p_under + (N_Bat_p_over - N_Bat_p_under)/3);
                    else
                        N_Bat_p = N_Bat_p_over;
                    end
                else
                    N_Bat_p = round(N_Bat_p_over - (N_Bat_p_over - N_Bat_p_under)/3);
                end
            else
                if (exist('N_Bat_p_over', 'var') == 1)
                    N_Bat_p = round(N_Bat_p_over*0.75);
                    if N_Bat_p == N_Bat_p_over
                        N_Bat_p = N_Bat_p_over - 1;
                    end
                else
                    N_Bat_p = round(N_Bat_p_under*5);
                    if N_Bat_p == N_Bat_p_under;
                        N_Bat_p = N_Bat_p_under + 1;
                    end
                end
            end
            if (exist('N_Bat_p_over', 'var') == 1)
                M_E_P(i_ite,4:5) = [N_Bat_p_under N_Bat_p_over];
                if ((N_Bat_p_over - 1) == N_Bat_p_under) && (N_Bat_p == N_Bat_p_over)
                    Bat_ok = 1; %Stop searching for N_Bat_p
                end
            else
                M_E_P(i_ite,4) = N_Bat_p_under;
            end
            %disp(M_E_P(i_ite,:))
            i_ite = i_ite + 1;
        end
        if N_Bat_p_pre ~= N_Bat_p
            for i=1:3
                design
                switch vehicle
                    case 'BEV'
                        sim('BEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
                    case 'FCHEV'
                        sim('FCHEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
                end
                PowerEnergyCalculation
                M_E_P(i_ite,:) = [N_Bat_p min(SoC_Bat) max(i_Bat/N_Bat_p) N_Bat_p_under N_Bat_p_over];
            end
        end
        clear N_Bat_p_under; clear N_Bat_p_over;
        
        switch vehicle
            case 'BEV'
                FC_ok = 1;
            case 'FCHEV'
                M_eval(i_ite_FC,:)  = [N_FC_p N_Bat_p E_H2/1000 M_car/1000];
                [N_M_eval,~]        = size(M_eval);
                n_temp = find(M_eval(:,3) == min(M_eval(:,3)));
                %                 if N_M_eval == n_temp(1)
                %                     N_FC_p      = N_FC_p + 1;
                %                     i_ite_FC    = i_ite_FC + 1;
                %                 else
                %                     FC_ok = 1;
                %                 end
                if P_FC_rat < P_FC_av
                    N_FC_p = N_FC_p + 1;
                else
                    FC_ok = 1;
                end
        end
    end
    switch vehicle
        case 'FCHEV'
            N_FC_p  = M_eval(n_temp(1),1);
            N_Bat_p = M_eval(n_temp(1),2);
            clear n_temp;
    end
    for i=1:3
        design
        switch vehicle
            case 'BEV'
                sim('BEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
            case 'FCHEV'
                sim('FCHEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
        end
        PowerEnergyCalculation
    end
    
    [T_s_sim, T_Cycle_start, T_Cycle_stop, T_start, T_stop, t_dc, v_car_dc, d_v_car_dc_dt] = number_of_dc(t, v_car, N_dc_design);
    switch vehicle
        case 'BEV'
            sim('BEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
        case 'FCHEV'
            sim('FCHEV', [T_start T_stop], simset('solver', 'ode23t', 'MaxStep', 0.1, 'MaxConsecutiveMinStep', 10));
    end
    PowerEnergyCalculation
    
    T_charging      = max(t_sim) - 3600 - T_Cycle_stop;      %[s] Time used for charging
    
    switch vehicle
        case 'BEV'
            E_all( 1,N_dc_design) = E_Grid/1000;                                         %[kWh] Energy consumption of "Tank"
            E_all(11,N_dc_design) = E_Loss_RF/1000;                                      %[kWh] Loss in rectifier
            M_temp = [M_car M_glider M_person M_EM M_Inv M_Bat M_BC M_BR M_RF];
            Mass_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                             %[kg] Total mass
            M_temp = [Vol_PS Vol_EM Vol_Inv Vol_Bat Vol_BC Vol_BR Vol_RF];
            Vol_PS_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                           %[L] Volume of power system
            M_temp = [Cost_car Cost_glider Cost_EM Cost_Inv Cost_Bat Cost_BC Cost_BR Cost_RF];
            Cost_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                             %[$] Total cost
        case 'FCHEV'
            E_all( 1,N_dc_design) = E_H2/1000;                                           %[kWh] Energy consumption of "Tank"
            E_all(11,N_dc_design) = E_Loss_FC/1000;                                      %[kWh] Loss in fuel cell
            M_temp = [M_car M_glider M_person M_EM M_Inv M_Bat M_BC M_BR M_FC M_H2 M_sto];
            Mass_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                             %[kg] Total mass
            M_temp = [Vol_PS Vol_EM Vol_Inv Vol_Bat Vol_BR Vol_BC Vol_FC Vol_sto];
            Vol_PS_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                           %[L] Volume of power system
            M_temp = [Cost_car Cost_glider Cost_EM Cost_Inv Cost_Bat Cost_BC Cost_BR Cost_FC Cost_sto];
            Cost_all(1:length(M_temp),N_dc_design) = M_temp'; clear M_temp;                             %[$] Total cost
    end
    E_all( 2,N_dc_design) = E_t/1000;                  %[kWh] Traction energy
    E_all( 3,N_dc_design) = E_MB/1000;                 %[kWh] Loss in mechanical brake
    E_all( 4,N_dc_design) = E_Loss_TS/1000;            %[kWh] Loss in transmission system
    E_all( 5,N_dc_design) = E_Loss_EM/1000;            %[kWh] Loss in electric machine
    E_all( 6,N_dc_design) = E_Loss_Inv/1000;           %[kWh] Loss in inverter
    E_all( 7,N_dc_design) = E_BR/1000;                 %[kWh] Loss in braking resistor
    E_all( 8,N_dc_design) = E_Loss_BC/1000;            %[kWh] Loss in boost converter
    E_all( 9,N_dc_design) = E_Aux/1000;                %[kWh] Loss due to auxiliary devices
    E_all(10,N_dc_design) = E_Loss_Bat/1000;           %[kWh] Loss in battery
    
    [N_dim,~] = size(E_all);
    eta_all(1:N_dim,N_dc_design) = 100*E_all(:,N_dc_design)/E_all(1,N_dc_design);
    clear N_dim;
    FC_car(1,N_dc_design) = E_all(1,N_dc_design)*1000/(N_dc_design*D_max(4));  %[Wh/km] Tank energy consumption over km - ratio
    
    
    disp('---------------------------------------------------')
    disp('Mass distribution [kg]:')
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Glider',                      Mass_all( 2,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Person',                      Mass_all( 3,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Electric machine',            Mass_all( 4,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Inverter',                    Mass_all( 5,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Battery',                     Mass_all( 6,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Boost converter',             Mass_all( 7,N_dc_design))
    switch vehicle
        case 'BEV'
            fprintf('%-40s %10.2f\n', 'Rectifier',           Mass_all( 8,N_dc_design))
        case 'FCHEV'
            fprintf('%-40s %10.2f\n', 'Fuel cell',           Mass_all( 8,N_dc_design))
            fprintf('%-40s %10.2f\n', 'Hydrogen',            Mass_all( 9,N_dc_design))
            fprintf('%-40s %10.2f\n', 'Hydrogen storage',    Mass_all(10,N_dc_design))
    end
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Car (sum)',                   Mass_all( 1,N_dc_design))
    disp('---------------------------------------------------')
    disp('---------------------------------------------------')
    disp('Volume distribution of power system [L]:')
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Electric machine',            Vol_PS_all( 2,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Inverter',                    Vol_PS_all( 3,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Battery',                     Vol_PS_all( 4,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Boost converter',             Vol_PS_all( 5,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Braking resistor',            Vol_PS_all( 6,N_dc_design))
    switch vehicle
        case 'BEV'
            fprintf('%-40s %10.2f\n', 'Rectifier',           Vol_PS_all( 7,N_dc_design))
        case 'FCHEV'
            fprintf('%-40s %10.2f\n', 'Fuel cell',           Vol_PS_all( 7,N_dc_design))
            fprintf('%-40s %10.2f\n', 'Hydrogen storage',    Vol_PS_all( 8,N_dc_design))
    end
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Power system (sum)',          Vol_PS_all( 1,N_dc_design))
    disp('---------------------------------------------------')
    disp('---------------------------------------------------')
    disp('Energy distribution [kWh]:')
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Traction E_t',                             E_all( 2,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Mechanical brake E_MB',                    E_all( 3,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Loss in transmission system E_Loss,TS',    E_all( 4,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Loss in electric machine E_Loss,EM',       E_all( 5,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Loss in inverter E_Loss,Inv',              E_all( 6,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Braking resistor E_BR',                    E_all( 7,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Loss in boost converter E_Loss,BC',        E_all( 8,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Auxiliary devices E_Aux',                  E_all( 9,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Loss in battery E_Loss,Bat',               E_all(10,N_dc_design))
    switch vehicle
        case 'BEV'
            fprintf('%-40s %10.2f\n', 'Loss in rectifier E_Loss,RF',      E_all(11,N_dc_design))
            disp('---------------------------------------------------')
            fprintf('%-40s %10.2f\n', 'Grid (sum) E_Grid',                E_all( 1,N_dc_design))
        case 'FCHEV'
            fprintf('%-40s %10.2f\n', 'Loss in fuel cell E_Loss,FC',      E_all(11,N_dc_design))
            disp('---------------------------------------------------')
            fprintf('%-40s %10.2f\n', 'Hydrogen (sum) E_H2',              E_all( 1,N_dc_design))
    end
    disp('---------------------------------------------------')
    disp('---------------------------------------------------')
    disp('Cost distribution [$]:')
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Glider',                     Cost_all(2,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Electric machine',           Cost_all(3,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Inverter',                   Cost_all(4,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Battery',                    Cost_all(5,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Braking resistor',           Cost_all(6,N_dc_design))
    fprintf('%-40s %10.2f\n', 'Boost converter',            Cost_all(7,N_dc_design))
    switch vehicle
        case 'BEV'
            fprintf('%-40s %10.2f\n', 'Rectifier',          Cost_all(8,N_dc_design))
        case 'FCHEV'
            fprintf('%-40s %10.2f\n', 'Fuel cell',          Cost_all(8,N_dc_design))
            fprintf('%-40s %10.2f\n', 'Hydrogen storage',   Cost_all(9,N_dc_design))
    end
    disp('---------------------------------------------------')
    fprintf('%-40s %10.2f\n', 'Car (sum)',                  Cost_all(1,N_dc_design))
    disp('---------------------------------------------------')
    disp('-------------------------------------------------------------------------------------------------')
    switch vehicle
        case 'BEV'
            fprintf('%-80s %10.2f %-s\n', 'Efficiency (traction energy relative to the grid energy): ',                         eta_all(2,N_dc_design), '%')
            fprintf('%-80s %10.2f %-s\n', 'Energy per km (grid energy/driving distance): ',                                     FC_car(1,N_dc_design), 'Wh/km')
        case 'FCHEV'
            fprintf('%-80s %10.2f %-s\n', 'Efficiency (traction energy relative to the energy of the consumed hydrogen): ',    eta_all(2,N_dc_design), '%')
            fprintf('%-80s %10.2f %-s\n', 'Energy per km (energy of consumed hydrogen/driving distance): ',                     FC_car(1,N_dc_design), 'Wh/km')
    end
    disp('-------------------------------------------------------------------------------------------------')
    
    figure(1+N_dc_design)
    [N_dim,~] = size(eta_all);
    eta_temp = zeros(N_dim-1,1);
    for n_dim=2:N_dim
        eta_temp(n_dim-1,1) = eta_all(n_dim,N_dc_design);
        if eta_temp(n_dim-1,1) == 0
            eta_temp(n_dim-1,1) = 1e-9;
        end
    end
    switch vehicle
        case 'BEV'
            pie(eta_temp,{['E_t: ', num2str(round(eta_temp(1))), ' %'], ['E_{MB}: ', num2str(round(eta_temp(2))), ' %'], ['E_{Loss,TS}: ', num2str(round(eta_temp(3))), ' %'],...
                ['E_{Loss,EM}: ', num2str(round(eta_temp(4))), ' %'], ['E_{Loss,Inv}: ', num2str(round(eta_temp(5))), ' %'], ['E_{BR}: ', num2str(round(eta_temp(6))), ' %'],...
                ['E_{Loss,BC}: ', num2str(round(eta_temp(7))), ' %'], ['E_{Aux}: ', num2str(round(eta_temp(8))), ' %'], ['E_{Loss,Bat}: ', num2str(round(eta_temp(9))), ' %'],...
                ['E_{Loss,RF}: ', num2str(round(eta_temp(10))), ' %']})
            %        print(1+N_dc_design, '-depsc2', '-r200', ['../report/gfx/Energy_pie_BEV_' num2str(N_dc_design) '.eps'])
            %Make eps-files of simulink models
            % print -deps -r300 -sBEV ../report/gfx/BEV_simulink_model.eps
        case 'FCHEV'
            pie(eta_temp,{['E_t = ', num2str(round(eta_temp(1))), ' %'], ['E_{MB}: ', num2str(round(eta_temp(2))), ' %'], ['E_{Loss,TS}: ', num2str(round(eta_temp(3))), ' %'],...
                ['E_{Loss,EM}: ', num2str(round(eta_temp(4))), ' %'], ['E_{Loss,Inv}: ', num2str(round(eta_temp(5))), ' %'], ['E_{BR}: ', num2str(round(eta_temp(6))), ' %'],...
                ['E_{Loss,BC}: ', num2str(round(eta_temp(7))), ' %'], ['E_{Aux}: ', num2str(round(eta_temp(8))), ' %'], ['E_{Loss,Bat}: ', num2str(round(eta_temp(9))), ' %'],...
                ['E_{Loss,FC}: ', num2str(round(eta_temp(10))), ' %']})
            %        print(1+N_dc_design, '-depsc2', '-r200', ['../report/gfx/Energy_pie_FCHEV_' num2str(N_dc_design) '.eps'])
            %Make eps-files of simulink models
            % print -deps -r300 -sFCHEV ../report/gfx/FCHEV_simulink_model.eps
    end
end
toc